"""Convenience module that imports all TailwindBuilder methods."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/all.ipynb.

# %% auto 0
__all__ = ['test_all_usage_examples']

# %% ../nbs/all.ipynb 3
# Import the TailwindBuilder class
from .core import TailwindBuilder

# Import all patched methods from each module
# This ensures TailwindBuilder has all methods available
from .layout import *
from .style import *
from .variants import *
from .modern import *

# %% ../nbs/all.ipynb 7
def test_all_usage_examples():
    """
    Test and demonstrate comprehensive TailwindBuilder usage with all modules.
    
    This function serves as both a test suite and a comprehensive usage guide
    for using TailwindBuilder with all methods available from all modules.
    It demonstrates how the 'all' module provides convenient access to the
    complete TailwindBuilder API.
    
    Rather than duplicating all tests, this function runs the test functions
    from each individual module to ensure comprehensive coverage.
    """
    
    # Import test functions from all modules
    from cjm_tailwind_utils.core import test_core_usage_examples
    from cjm_tailwind_utils.colors import test_colors_usage_examples
    from cjm_tailwind_utils.validation import test_validation_usage_examples
    from cjm_tailwind_utils.layout import test_layout_usage_examples
    from cjm_tailwind_utils.variants import test_variants_usage_examples
    from cjm_tailwind_utils.style import test_style_usage_examples
    from cjm_tailwind_utils.modern import test_modern_usage_examples
    
    print("=== Testing Core Functionality ===")
    print(test_core_usage_examples())
    print()
    
    print("=== Testing Color System ===")
    print(test_colors_usage_examples())
    print()
    
    print("=== Testing Validation System ===")
    print(test_validation_usage_examples())
    print()
    
    print("=== Testing Layout Utilities ===")
    print(test_layout_usage_examples())
    print()
    
    print("=== Testing Variant Utilities ===")
    print(test_variants_usage_examples())
    print()
    
    print("=== Testing Style Utilities ===")
    print(test_style_usage_examples())
    print()
    
    print("=== Testing Modern CSS Features ===")
    print(test_modern_usage_examples())
    print()
    
    # === Comprehensive Integration Example ===
    print("=== Integration Example: Building a Complete Component ===")
    
    # Example: Building a modern card component with all features
    card_builder = TailwindBuilder(dedupe=True)
    
    # Structure and spacing
    card_classes = (card_builder
        # Container with responsive padding
        .p(4).sm("p-6").lg("p-8")
        # Flexbox layout
        .flex().flex("col").gap(4)
        # Sizing
        .w("full").max_w("2xl")
        # Background and borders
        .bg("white").dark("bg-gray-900")
        .border(2, color="gray-200").dark("border-gray-700")
        .rounded("xl")
        # Shadow and effects
        .shadow("lg").hover("shadow-xl")
        # Transitions
        .transition("all", duration=300)
        # Modern features
        .container_query()
        .cq("sm", "grid-cols-2")
        .has("[data-featured]", "ring-2", "ring-blue-500")
        .build()
    )
    
    print(f"Card component classes:\n{card_classes}")
    
    # Example: Building an interactive button
    button_builder = TailwindBuilder(dedupe=True)
    
    button_classes = (button_builder
        # Base styles
        .p(3, "x").p(2, "y")
        .bg("blue-600").hover("bg-blue-700")
        .text(color="white", weight="medium")
        .rounded("md")
        # States
        .focus("outline-none", "ring-2", "ring-blue-500", "ring-offset-2")
        .active("scale-95")
        .disabled("opacity-50", "cursor-not-allowed")
        # Transitions
        .transition(duration=150)
        # Dark mode
        .dark("bg-blue-500", "hover:bg-blue-600")
        .build()
    )
    
    print(f"\nButton component classes:\n{button_classes}")
    
    return "All module tests completed successfully!"
